//
// soaputil.cpp
//
//	Written by Chris Dix, July 2000
//  Don't drop the SOAP.

#include <stdio.h>
#include <string.h>
#include <iostream.h>

#include <winsock2.h>   // Windows socket functions

#include "soaputil.h"
using namespace SoapUtils;

SimpleSoapMessage::SimpleSoapMessage() : 
    m_strPrefix( "SOAP-ENV" ),
    m_bContainsFault( false ),
    m_bIsValid( false )
{
}

SimpleSoapMessage::~SimpleSoapMessage()
{
}

void SimpleSoapMessage::clear( void )
{
    m_bIsValid = false;
    m_bContainsFault = false;

    m_strPrefix.erase();
    m_strBody.erase();
    
    m_strError.erase();
}

bool SimpleSoapMessage::putText( const string& strText )
{
    if ( parse( strText ) )
        return( true );
    clear();
    return( false );
}

string SimpleSoapMessage::getText( void ) const
{
    string strMsg;
    bool bPrefix = !m_strPrefix.empty();

    strMsg = "<?xml version=\"1.0\"?><";
    strMsg += m_strPrefix;
    if ( bPrefix )
        strMsg += ":";
    strMsg += "Envelope xmlns";
    if ( bPrefix )
    {
        strMsg += ":";
    }
    strMsg += m_strPrefix;
    strMsg += "='";
    strMsg += strVersion;
    strMsg += "' ";
    strMsg += m_strPrefix;
    if ( bPrefix )
    {
        strMsg += ":";
    }
    strMsg += "encodingStyle=";
    strMsg +=
        "'http://schemas.xmlsoap.org/soap/encoding/'>";
    strMsg += "<";
    strMsg += m_strPrefix;
    if ( bPrefix )
    {
        strMsg += ":";
    }
    strMsg += "Body>";
    
    strMsg += getBody();

    strMsg += "</";
    strMsg += m_strPrefix;
    if ( bPrefix )
    {
        strMsg += ":";
    }
    strMsg += "Body>";
    strMsg += "</";
    strMsg += m_strPrefix;
    if ( bPrefix )
    {
        strMsg += ":";
    }
    strMsg += "Envelope>";

    return( strMsg );
}

void SimpleSoapMessage::putBody( const string& strBody )
{
    m_strBody = strBody;
}

string SimpleSoapMessage::getBody( void ) const
{
    return( string( m_strBody ) );
}

string SimpleSoapMessage::getError( void ) const
{
    return( string( m_strError ) );    
}

bool SimpleSoapMessage::extractElementContent( 
    const string& strText, const string& strName, 
    string& strElem )
{    
    string strTemp;
    string strEndTag;
    bool bSuccess = false;

    if ( strText.empty() ) return( false );
    if ( strName.empty() ) return( false );    

    strElem.erase();

    strEndTag = "</";
    strEndTag += strName;
    strEndTag += ">";

    int nStart = strText.find( strName );
    if ( nStart != string::npos )
    {
        string strStart = strText.substr( nStart );
        nStart = strStart.find( ">" );
        if ( nStart != string::npos )
        {
            strStart = strStart.substr( nStart + 1 );
            int nEnd = strStart.find( strEndTag );
            if ( nEnd != string::npos )
            {
                strElem = strStart.substr( 0, nEnd );
                bSuccess = true;
            }
        }
    }
    return( bSuccess );
}

bool SimpleSoapMessage::extractFirstElement( 
    const string& strElem, string& strName, 
    string& strChild )
{    
    string strStart;
    string strEnd;
    string strTest;
    bool bParsing = true;
    bool bEmpty = false;
    bool bSuccess = false;

    if ( strElem.empty() ) return( false );
    
    strStart = strElem;

    // Skip the XML declaration, comments, etc.
    while ( bParsing )
    {
        int nStart = strStart.find( "<" );
        if ( nStart != string::npos )
        {
            strStart = strStart.substr( nStart + 1 );
            if ( ( strStart.at( 0 ) != '!' ) &&
                 ( strStart.at( 0 ) != '?' ) )
            {
                bParsing = false;
            }
        }
        else
        {
            bParsing = false;
        }
    }
    
    if ( strStart.empty() ) return( false );

    // Find the full name
    int nEnd = strStart.find_first_of( " />" );
    if ( nEnd != string::npos )
    {
        strEnd = strStart.substr( nEnd );
        if ( strEnd.at( 0 ) == '/' )
            bEmpty = true;

        strName = strStart.substr( 0, nEnd );
    }

    if ( bEmpty )
        return( true );

    return( extractElementContent( strElem, strName, 
        strChild ) );
}

void SimpleSoapMessage::splitElementName( 
    const string& strName, string& strPrefix, 
    string& strBasename )
{
    if ( strName.empty() ) return;    

    int nColon = strName.find( ":" );
    if ( nColon != string::npos )
    {
        int nCopyLen = strName.length() - nColon;
        strPrefix = strName.substr( 0, nColon );
        
        nColon++;
        strBasename = strName.substr( nColon,
            nCopyLen );
    }
    else
    {
        strBasename = strName;
    }
}

bool SimpleSoapMessage::parse( const string& strMsg )
{
    string strEnvelope;
    string strSearch;
    string strTemp;
    string strStart;
    string strEnd;
    bool bParsing = true;
    bool bSuccess = false;
    bool bPrefix = true;

    clear();

    if ( strMsg.empty() ) return( false );

    strSearch = "Envelope";
    bSuccess = extractFirstElement( strMsg, 
        strSearch, strEnvelope );
    if ( !bSuccess ) return( false );

    splitElementName( strSearch, m_strPrefix, strTemp );
    if ( m_strPrefix.empty() )
        bPrefix = false;

    string strBodyTag;
    string strFaultTag;

    if ( bPrefix )
    {
        strBodyTag = m_strPrefix + ":Body";
        strFaultTag = m_strPrefix + ":Fault";
    }
    else
    {
        strBodyTag = "Body";
        strFaultTag = "Fault";
    }

    bSuccess = extractElementContent( strMsg,
        strBodyTag, m_strBody );
    if ( !bSuccess ) return( false );

    m_bIsValid = true;

    m_bContainsFault = extractElementContent( m_strBody,
        strFaultTag, strTemp );
    
    return( m_bIsValid );
}

SimpleSoapRpcMessage::SimpleSoapRpcMessage()
{
}

SimpleSoapRpcMessage::~SimpleSoapRpcMessage()
{
}

string SimpleSoapRpcMessage::getBody( void ) const
{
    // If we haven't populated this object,
    // maybe it was a response.
    if ( m_strMethod.empty() )
        return( m_strBody );

    string strBody = "<";
    if ( m_strMethodNamespaceURI.length() )
    {
        strBody += m_strMethodNamespacePrefix;
        strBody += ":";
    }
    strBody += m_strMethod;
    if ( m_strMethodNamespaceURI.length() )
    {
        strBody += " xmlns:";
        strBody += m_strMethodNamespacePrefix;
        strBody += "='";
        strBody += m_strMethodNamespaceURI;
        strBody += "'";
    }

    if ( m_mapParams.size() <= 0 )
    {
        strBody += "/>";
        return( strBody );
    }
    strBody += ">";

    // Insert the parameters.
    striterator si;
    for ( si = m_mapParams.begin(); 
        si != m_mapParams.end(); si++ )
    {
        strBody += "<";
        strBody += si->first;
        strBody += ">";
        strBody += si->second;
        strBody += "</";
        strBody += si->first;
        strBody += ">";
    }

    strBody += "</";
    if ( m_strMethodNamespaceURI.length() )
    {
        strBody += m_strMethodNamespacePrefix;
        strBody += ":";
    }
    strBody += m_strMethod;
    strBody += ">";

    return( strBody );
}

void SimpleSoapRpcMessage::clear( void )
{
    m_strMethod.erase();
    m_strMethodNamespaceURI.erase();
    m_mapParams.clear();
}

void SimpleSoapRpcMessage::addParameter( 
    const string& strParam,
    const string& strValue )
{
    m_mapParams[strParam] = strValue;
}

bool SimpleSoapRpcMessage::containsParameter( 
    const string& strParam )
{
    striterator i = m_mapParams.find( strParam );
    if ( i != m_mapParams.end() )
    {
        return( true );
    }
    return( false );
}

string SimpleSoapRpcMessage::getParameterValue( 
    const string& strParam )
{
    return( m_mapParams[strParam] );
}

void SimpleSoapRpcMessage::removeParameter( 
    const string& strParam )
{
    m_mapParams.erase( strParam );
}

bool SimpleSoapRpcMessage::parse( const string& strMsg )
{
    clear();

    if ( SimpleSoapMessage::parse( strMsg ) )
    {
        // NOTE: This is where, if we had an XML parser,
        // we would read the elements of the Body
        // and map them into the method name and the 
        // parameters.  This is left as an exercise for the
        // reader...
        return( true );
    }

    return( false );
}

string SimpleSoapRpcMessage::getReturnValue( void )
{
    string strName;
    string strContent;
    string strReturn;

    if ( !isValid() ) return( "" );
    
    if ( m_strBody.empty() ) return( strContent );
    
    if ( m_bContainsFault )
        return( m_strBody );

    bool bSuccess = extractFirstElement( m_strBody, 
        strName, strContent );
    if ( bSuccess )
    {
        bSuccess = extractFirstElement( strContent, 
            strName, strReturn );        
    }

    return( strReturn );
}

bool SoapHttpPostTransport::ProtocolInit( void )
{
    WSADATA wsa;
    if ( WSAStartup( 0x0101, &wsa ) == -1 ) 
        return( false );    
    
    return( true );
}

void SoapHttpPostTransport::ProtocolShutdown( void )
{
    WSACleanup();
}

bool SoapHttpPostTransport::sendMessage( 
    const SimpleSoapMessage* preq,
    SimpleSoapMessage* presp )
{
    SOCKET          s;
    SOCKADDR_IN     sin;
    PHOSTENT        ph = NULL;
    int             nMax = nMaxLen;
    int             nOptval = 1;
    bool            bContinue = true;
    bool            bSuccess = false;

    // Check for valid messages.
    if ( preq == NULL ) return( false );
    if ( presp == NULL ) return( false );

    if ( !ProtocolInit() )
    {
        cout << "Could not initialize socket library.\n";
        return( false );
    }
    
    s = socket( AF_INET, SOCK_STREAM, 0 );
    if ( s == INVALID_SOCKET ) 
    {
        cout << "Could not open socket.\n";
        return( false );
    }

    // Allow reuse of local address
    memset( &sin, 0, sizeof(sin) );
    if ( setsockopt( s, SOL_SOCKET, SO_REUSEADDR, 
       (char*)(&nOptval), sizeof(nOptval) ) == SOCKET_ERROR )
    {
        cout << "Could not reuse local address.\n";
        return( false );
    }

    sin.sin_family = AF_INET;
    sin.sin_port = htons(m_uiPort);

    ph = gethostbyname( m_strHost.c_str() );
    if ( ph == NULL )
    {
        cout << "Could not locate endpoint host.\n";
        return( false );
    }

    memcpy( &(sin.sin_addr), ph->h_addr_list[0], 
        ph->h_length );

    if ( connect( s, (struct sockaddr*)(&sin),
        sizeof( sin ) ) == SOCKET_ERROR )
    {
        cout << "Failed to connect to endpoint host.\n";
        return( false );
    }
    
    // Build the request
    string strBuffer;
    string strMsg;
    string strHttpResponse;
    string strSoapResponse;
    string strRead;
    string strEnd;
    string strCode;

    char* pszLen = new char[5];

    strMsg = preq->getText();

    // We need to include the message length as an 
    // HTTP header
    memset( pszLen, 0, 5 );
    itoa( strMsg.length(), pszLen, 10 );

    // Build the necessary HTTP headers.
    strBuffer = "POST ";
    strBuffer += m_strAddress;
    strBuffer += " HTTP/1.1";
    strBuffer += "\r\n";
    strBuffer += "Host: ";
    strBuffer += m_strHost;
    strBuffer += "\r\n";
    strBuffer += "Content-Type: text/xml";
    strBuffer += "\r\n";
    strBuffer += "Content-Length: ";
    strBuffer += pszLen;  
    strBuffer += "\r\n";
    strBuffer += "SOAPAction: \"";
    strBuffer += m_strAction;
    
    strBuffer += "\"";
    strBuffer += "\r\n";
    strBuffer += "\r\n";
    strBuffer += strMsg;

    if ( m_bDebugging )
    {
        cout << "Sending message...\n";
        //cout << strBuffer.c_str();
        //cout << "\n";
    }
    
    // Get the length of the HTTP POST 
    int nBufferLen = strBuffer.length();
    int nSize = 0;

    if ( send( s, strBuffer.c_str(), 
        nBufferLen, 0 ) == nBufferLen )
    {
        char* pszHttpResponse = new char[nMaxLen];
        int nEnd;
        int nSoapResponse;

        while ( bContinue )
        {
            memset( pszHttpResponse, 0, nMaxLen );
            int nResponseLen = recv( s, 
                pszHttpResponse, nMax, 0 );
            if ( nResponseLen == SOCKET_ERROR ) 
                return( false );

            strHttpResponse = pszHttpResponse;
            
            // If we get a HTTP continue, then we may 
            // need to  keep reading on this socket.
            if ( strHttpResponse.compare( 0,
                strHttpContinue.length(),
                strHttpContinue ) == 0 )
            {
                nEnd = strHttpResponse.find( 
                    "\r\n\r\n" );
                if ( nEnd != string::npos )
                {
                    nEnd += 4;
                    strRead = strHttpResponse.substr( nEnd );
                    
                    nEnd = strRead.find( "HTTP/1.1" );
                    if ( nEnd != string::npos )
                        bContinue = false;
                }
            }
            else
            {
                strRead = strHttpResponse;
                bContinue = false;
            }
        }

        bool bShowHttpCode = false;

        if ( m_bDebugging )
        {
            cout << "Received response...\n";
            //cout << strRead.c_str();
            //cout << "\n";
        }
    
        // 200 = valid response
        // 500 = SOAP Fault
        // Both return the response
        bool bOK = ( strRead.compare( 0, 
            strHttpOK.length(), 
            strHttpOK ) == 0 ? true : false );
        bool bFault = ( strRead.compare( 0, 
            strHttpFault.length(), 
            strHttpFault ) == 0 ? true : false );

        if ( bOK || bFault )
        {
            // We have received a valid message; now we 
            // need to process the headers.
            char* pszHeader = NULL;
            bool bXML = false;

            // Make sure it's text/xml
            int nHeader = strRead.find( strTypeHeader );
            if ( nHeader != string::npos )
            {
                bXML = true;
            }
            else
            {
                if ( m_bDebugging )
                    cout << "Non-XML response.\n";
            }

            nSoapResponse = strRead.find( "\r\n\r\n" );
            if ( nSoapResponse )
            {
                nSoapResponse += 4;
                strSoapResponse = strRead.substr( 
                    nSoapResponse );
                bSuccess = presp->putText( strSoapResponse );
            }
            else
            {
                bShowHttpCode = true;
            }
        }
        else
        {
            bShowHttpCode = true;
        }

        if ( bShowHttpCode && m_bDebugging )
        {
            // We got some other status code, 
            // print it for debugging.
            cout << "HTTP Error\n";

            nEnd = strRead.find( "\r\n" );
            if ( nEnd != string::npos )
            {
                string strCode = strRead.substr( 0, nEnd );
                cout << strCode.c_str();
                cout << "\n";
            }
        }

        delete pszHttpResponse;
    }

    delete pszLen;

    return( bSuccess );
}
